// flex-buffer.js

module.exports = FlexBuffer;

var utils = require("./flex-utils");

function FlexBuffer() {
  if (!(this instanceof FlexBuffer)) return new FlexBuffer();
}

var methods = {
  write: write,
  fetch: fetch,
  flush: flush,
  push: push,
  pull: pull,
  read: read,
  offset: 0
};

var mixin = FlexBuffer.mixin = utils.createMixin(methods);

mixin(FlexBuffer.prototype);

function write() {
  throw new Error("method not implemented: write()");
}

function fetch() {
  throw new Error("method not implemented: fetch()");
}

function flush() {
  throw new Error("method not implemented: flush()");
}

function read() {
  var length = this.buffers && this.buffers.length;

  // fetch the first result
  if (!length) return this.fetch();

  // flush current buffer
  this.flush();

  // read from the results
  return this.pull();
}

function push(chunk) {
  var buffers = this.buffers || (this.buffers = []);
  buffers.push(chunk);
}

function pull() {
  var buffers = this.buffers || (this.buffers = []);
  return buffers.shift();
}
