// flex-encoder.js

module.exports = FlexEncoder;

var FlexBuffer = require("./flex-buffer");
var utils = require("./flex-utils");

var MIN_BUFFER_SIZE = 2048;
var MAX_BUFFER_SIZE = 65536;

function FlexEncoder() {
  if (!(this instanceof FlexEncoder)) return new FlexEncoder();
}

var methods = FlexBuffer.mixin({});
methods.fetch = fetch;
methods.pull = pull;
methods.reserve = reserve;
methods.send = send;
methods.start = 0;

var mixin = FlexEncoder.mixin = utils.createMixin(methods);

mixin(FlexEncoder.prototype);

function fetch() {
  var start = this.start;
  if (start < this.offset) {
    this.start = this.offset;
    return this.buffer.slice(start, this.offset);
  }
}

function pull() {
  var buffers = this.buffers || (this.buffers = []);
  var chunk = buffers.length > 1 ? Buffer.concat(buffers) : buffers[0];
  buffers.length = 0; // buffer exhausted
  return chunk;
}

function reserve(length) {
  if (this.buffer) {
    var size = this.buffer.length;

    // is it long enough?
    if (this.offset + length < size) return;

    // flush current buffer
    this.flush();

    // resize it to 2x current length
    length = Math.max(length, Math.min(size * 2, MAX_BUFFER_SIZE));
  }

  // minimum buffer size
  length = length > MIN_BUFFER_SIZE ? length : MIN_BUFFER_SIZE;

  // allocate new buffer
  this.buffer = new Buffer(length);
  this.start = 0;
  this.offset = 0;
}

function send(buffer) {
  var end = this.offset + buffer.length;
  if (this.buffer && end < this.buffer.length) {
    buffer.copy(this.buffer, this.offset);
    this.offset = end;
  } else {
    this.flush();
    this.push(buffer);
  }
}
